<?php

namespace App\Form;

use App\Entity\User;
use App\Repository\PendingEmailAddressRepository;
use App\Repository\UserRepository;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Security\Core\Validator\Constraints\UserPassword;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

class EditUserProfileFormType extends AbstractType
{
    private UserRepository $user_repository;
    private PendingEmailAddressRepository $pending_email_address_repository;

    public function __construct(
        UserRepository $user_repository,
        PendingEmailAddressRepository $pending_email_address_repository
    ) {
        $this->user_repository = $user_repository;
        $this->pending_email_address_repository = $pending_email_address_repository;
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $user_repository = $this->user_repository;
        $pending_email_address_repository = $this->pending_email_address_repository;

        $builder
            ->add(
                'firstName',
                TextType::class,
                [
                    'constraints' => [new NotBlank(['message' => 'firstName.not_blank'])],
                    'label' => 'user.firstname',
                    'empty_data' => ''
                ]
            )
            ->add(
                'lastName',
                TextType::class,
                [
                    'constraints' => [new NotBlank(['message' => 'lastName.not_blank'])],
                    'label' => 'user.lastname',
                    'empty_data' => ''
                ]
            )
            ->add(
                'email',
                EmailType::class,
                [
                    'data' => $options['current_email_address'],
                    'constraints' => [
                        new NotBlank(['message' => 'email.not_blank']),
                        new Assert\Email(),
                        new Assert\Callback(
                            ['callback' => static function (
                                string $value,
                                ExecutionContextInterface $context
                            ) use (
                                $options,
                                $user_repository,
                                $pending_email_address_repository
                            ) {
                                if (
                                    $value !== $options['current_email_address'] &&
                                    ($user_repository->emailExists($value) ||
                                        $pending_email_address_repository->emailIsReserved($value))
                                ) {
                                    $context
                                        ->buildViolation(message: 'user.email.unique')
                                        ->addViolation()
                                    ;
                                }
                            }
                            ]
                        )
                    ],
                    'label' => 'general.email',
                    'empty_data' => '',
                    'mapped' => false
                ]
            )
            ->add(
                'current_password',
                PasswordType::class,
                [
                    'mapped' => false,
                    'required' => true,
                    'constraints' => [
                        new NotBlank(['message' => 'password.not_blank']),
                        new UserPassword(['message' => "password.invalid"])
                    ],
                    'label' => 'user.password.current'
                ]
            )
            ->add(
                'update',
                SubmitType::class,
                ['label' => 'user.profile.update']
            )
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => User::class,
            'current_email_address' => '',
            'users_emails' => [],
            'pending_emails' => []
            ]);
    }
}
